@@ -23,6 +23,9 @@ pub fn get_available_encodings() -> &'static [&'static str] {
     #[allow(static_mut_refs)]
     unsafe {
         if ENCODINGS.is_empty() {
+            ENCODINGS.push("UTF-8");
+            ENCODINGS.push("UTF-8 BOM");
+
             if let Ok(f) = init_if_needed() {
                 let mut n = 0;
                 loop {
@@ -30,14 +33,17 @@ pub fn get_available_encodings() -> &'static [&'static str] {
                     if name.is_null() {
                         break;
                     }
-                    ENCODINGS.push(CStr::from_ptr(name).to_str().unwrap_unchecked());
+
+                    let name = CStr::from_ptr(name).to_str().unwrap_unchecked();
+                    // We have already pushed UTF-8 above.
+                    // There is no need to filter UTF-8 BOM here, since ICU does not distinguish it from UTF-8.
+                    if name != "UTF-8" {
+                        ENCODINGS.push(name);
+                    }
+
                     n += 1;
                 }
             }
-
-            if ENCODINGS.is_empty() {
-                ENCODINGS.push("UTF-8");
-            }
         }
         &ENCODINGS
     }
@@ -622,17 +628,25 @@ impl Regex {
     /// # Safety
     ///
     /// The caller must ensure that the given `Text` outlives the `Regex` instance.
-    pub unsafe fn set_text(&mut self, text: &Text) {
+    pub unsafe fn set_text(&mut self, text: &mut Text, offset: usize) {
+        // Get `utext_access_impl` to detect the `TextBuffer::generation` change,
+        // and refresh its contents. This ensures that ICU doesn't reuse
+        // stale `UText::chunk_contents`, as it has no way tell that it's stale.
+        utext_access(text.0, offset as i64, true);
+
         let f = assume_loaded();
         let mut status = icu_ffi::U_ZERO_ERROR;
         unsafe { (f.uregex_setUText)(self.0, text.0 as *const _ as *mut _, &mut status) };
+        // `uregex_setUText` resets the regex to the start of the text.
+        // Because of this, we must also call `uregex_reset64`.
+        unsafe { (f.uregex_reset64)(self.0, offset as i64, &mut status) };
     }
 
     /// Sets the regex to the absolute offset in the underlying text.
-    pub fn reset(&mut self, index: usize) {
+    pub fn reset(&mut self, offset: usize) {
         let f = assume_loaded();
         let mut status = icu_ffi::U_ZERO_ERROR;
-        unsafe { (f.uregex_reset64)(self.0, index as i64, &mut status) };
+        unsafe { (f.uregex_reset64)(self.0, offset as i64, &mut status) };
     }
 }
 
